<?php
/* * *************************************************************************
 *
 *   PROJECT: Affiliate APP
 *   powerd by ashraf hamdy
 *   Copyright 2024 IT Plus Inc
 *   http://it-plus.co/
 *
 *
 * ************************************************************************* */
class iplus {

    private string $mDbhost;
    private string $mDbuser;
    private string $mDbpwd;
    private string $mDbname;
    private Mailer $mMailer;
    private array $mConfig;
    private PDO $mDb; // إضافة الخاصية لحفظ الاتصال

    public function __construct() {
        global $Config;
        $this->mDbhost = $Config['dbhost'];
        $this->mDbuser = $Config['dbuser'];
        $this->mDbpwd = $Config['dbpwd'];
        $this->mDbname = $Config['dbname'];
        $this->mMailer = new Mailer();
        $this->mConfig = $Config;
    }

    /**
     * إنشاء أو إرجاع اتصال قاعدة البيانات
     * 
     * @return PDO كائن PDO للاتصال بقاعدة البيانات
     * @throws PDOException في حالة فشل الاتصال بقاعدة البيانات
     * 
     * الدالة تقوم بما يلي:
     * - التحقق من وجود اتصال سابق
     * - إنشاء اتصال جديد إذا لم يكن موجوداً
     * - ضبط إعدادات الاتصال (UTF-8 وإظهار الأخطاء)
     * - تخزين الاتصال في المتغير mDb لإعادة استخدامه
     */
    public function connect(): PDO {
        if (isset($this->mDb)) {
            return $this->mDb; // إرجاع كائن PDO المخزن مسبقا
        }
        $dsn = "mysql:host=$this->mDbhost;dbname=$this->mDbname;";
        $user = $this->mDbuser;
        $pass = $this->mDbpwd;
        $option = array(
            PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES utf8',
        );
        try {
            $this->mDb = new PDO($dsn, $user, $pass, $option);
            $this->mDb->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            return $this->mDb; // إرجاع كائن PDO بعد الاتصال الناجح
        } catch (PDOException $e) {
            echo "Failed To Connect " . $e->getMessage();
        }
    }

    public function query($aSql) {
        $query = $this->connect()->prepare($aSql);
        return $query->execute();
    }

    public function queryReturnRowCount($aSql) {
        $query = $this->connect()->prepare($aSql);
        $query->execute();
        return $query->rowCount();
    }

     function queryreturnlastid($aSql) {
        $con = $this->connect();
        $query = $con->prepare($aSql);
        $query->execute();
        return $con->lastInsertId();
    }

    public function getRow($aSql) {
        $query = $this->connect()->prepare($aSql);
        $query->execute();
        return $query->fetch(PDO::FETCH_ASSOC);
    }
    public function getAll($aSql) {
        $query = $this->connect()->prepare($aSql);
        $query->execute();
        $temp = $query->fetchAll(PDO::FETCH_ASSOC);
        return $temp;
    }

    /**
     * استرجاع نتائج الاستعلام كمصفوفة ترابطية مع تجميع النتائج حسب العمود الأول
     *
     * @param string $aSql استعلام SQL المراد تنفيذه
     * @return array مصفوفة ترابطية مع تجميع النتائج حسب العمود الأول
     * @throws PDOException في حالة فشل تنفيذ الاستعلام
     */
    function &getAssoc($aSql) {
        try {
            $out = array();
            $stmt = $this->connect()->prepare($aSql);
            
            if (!$stmt->execute()) {
                throw new PDOException("فشل تنفيذ الاستعلام");
            }

            while ($temp = $stmt->fetch(PDO::FETCH_ASSOC)) {
                if (empty($temp)) {
                    break;
                }
                $key = array_shift($temp);
                if ($key !== null) {
                    $out[$key][] = $temp;
                }
            }
            
            return $out;
        } catch (PDOException $e) {
            error_log("Error in getAssoc: " . $e->getMessage());
            throw $e;
        }
    }

    function getOne($aSql) {

        $query = $this->connect()->prepare($aSql);
        $query->execute();
        return $result = $query->fetchColumn(0);
    }

    function getTables() {
        $out = array();

        $sql = "SHOW TABLES FROM {$this->mDbname}";
        $res = $this->connect()->prepare($sql);
        $res->execute();
        while ($row = $res->fetch()) {
            $out[] = $row[0];
        }
        return $out;
    }

    function printError($aError) {
        echo $aError;
    }

    function getCols($tableName, $Req) {
        $aSql = "SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = N'" . $tableName . "' and TABLE_SCHEMA = '" . $this->mDbname . "' ";
        $query = $this->connect()->prepare($aSql);
        $query->execute();
        $out = array();
        while ($temp = $query->fetch(PDO::FETCH_ASSOC)) {
            if ($temp['COLUMN_NAME'] != 'id' && $temp['COLUMN_NAME'] != 'id') {
                if (isset($Req[$temp['COLUMN_NAME']])) {
                    $return[$temp['COLUMN_NAME']] = $Req[$temp['COLUMN_NAME']];
                }
            }
        }

        return $return;
    }

    function getColumnName($tableName) {
        $aSql = "SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = N'" . $tableName . "' and TABLE_SCHEMA = '" . $this->mDbname . "' ";
        $aSql.="  ORDER BY `ORDINAL_POSITION` ASC ";
        $query = $this->connect()->prepare($aSql);
        $query->execute();
        $return = array();

        while ($temp = $query->fetch(PDO::FETCH_ASSOC)) {
            if ($temp['COLUMN_NAME'] != 'id' && $temp['COLUMN_NAME'] != 'admin_id' && $temp['COLUMN_NAME'] != 'date_added') {

                $arr = explode("_", $temp['COLUMN_NAME']);
                $arrpop = $arr;
                array_pop($arrpop);
                $arrpop = implode("_", $arrpop);
                $i = array_search(reset($arr), array_column($return, 'mode'));

                if ($i > -1 && end($arr) !== 'show') {

                    $i3 = array_search($arrpop, array_column($return[$i]['options'], 'mode'));
                    if ($i3 > -1) {
                        $return[$i]['options'][$i3][$temp['COLUMN_NAME']] = $temp['COLUMN_DEFAULT'];
                    } else {
                        $options = array();
                        $options['mode'] = $arrpop;
                        $options[$temp['COLUMN_NAME']] = $temp['COLUMN_DEFAULT'];
                        array_push($return[$i]['options'], $options);
                    }
                } else {
                    if (end($arr) == 'show') {
                        $i2 = array_search(reset($arr), array_column($return, 'mode'));
                        if ($i2 > -1) {
                            $return[$i2][$temp['COLUMN_NAME']] = $temp['COLUMN_DEFAULT'];
                        } else {
                            array_push($return, ["mode" => reset($arr), $temp['COLUMN_NAME'] => $temp['COLUMN_DEFAULT'], "options" => array()]);
                        }
                    } else {
                        array_push($return, ["mode" => reset($arr), $temp['COLUMN_NAME'] => $temp['COLUMN_DEFAULT'], "options" => array()]);
                    }
                }
            }
        }

        return $return;
    }

    function getKeyValue($aSql) {
        $out = array();
        $res = $this->connect()->prepare($aSql);
        $res->execute();
        while ($temp = $res->fetch()) {
            $out[$temp[0]] = $temp[1];
        }
        return $out;
    }

    public function getLastInsertId() {
        return $this->connect()->lastInsertId();
    }

    public function getlastIdFromTable($table) {
        $sql = " SELECT `AUTO_INCREMENT` FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = '{$table}' ";
        $res = $this->getOne($sql);
        if ($res) {
            return $res;
        } else {
            return 0;
        }
    }

    public function autoIncrementTableName($tableName) {
        $sql = " SELECT MAX(`id`) + 1 as count FROM `{$tableName}` ";
        $count = $this->getOne($sql);
        $count = $count ? $count : 0;
        $sql = " ALTER TABLE `{$tableName}` AUTO_INCREMENT ={$count} ";
        return $this->query($sql);
    }

    public function addCronJob($filename = '') {
        if ($filename) {
            $output = shell_exec('crontab -l');
            if (stripos($output, $filename) == false) {
                file_put_contents("/tmp/crontab.txt", $output . $filename . PHP_EOL);
                return exec('crontab  /tmp/crontab.txt');
            }
        }
    }

    public function delCronJob($filename = '') {
        if ($filename) {
            $output = shell_exec('crontab -l');
            $remove_cron = str_replace($filename, "", $output);
            file_put_contents("/tmp/crontab.txt", $remove_cron . PHP_EOL);
            return exec('crontab  /tmp/crontab.txt');
        }
    }

    // Add transaction support methods
    public function beginTransaction() {
        return $this->connect()->beginTransaction();
    }

    public function commit() {
        return $this->connect()->commit();
    }

    public function rollBack() {
        return $this->connect()->rollBack();
    }
}
